#include <webkit2/webkit2.h>

#include "glob-app-window.h"
#include "glob-search.h"
#include "glob-word.h"

struct _GlobAppWindow
{
    GtkApplicationWindow parent_instance;

    GlobSearch *search;
    GtkWidget  *header;
    GtkWidget  *webview;
    GtkWidget  *listbox;
};

G_DEFINE_TYPE (GlobAppWindow, glob_app_window, GTK_TYPE_APPLICATION_WINDOW)

static void
update_header (GlobSearch    *search,
               guint          matches,
               GlobAppWindow *win)
{
    gchar *status;

    if (matches == 1)
        status = g_strdup_printf ("Found 1 match");
    else
        status = g_strdup_printf ("Found %d matches", matches);

    gtk_header_bar_set_subtitle (GTK_HEADER_BAR (win->header), status);

    g_free (status);
}

static void
row_activated_cb (GtkListBox    *self,
                  GtkListBoxRow *row,
                  gpointer       user_data)
{
    GlobAppWindow *window = GLOB_APP_WINDOW (user_data);

    GtkWidget *label = gtk_bin_get_child (GTK_BIN (row));
    const gchar *word = gtk_label_get_label (GTK_LABEL (label));
    gchar *uri = g_strdup_printf ("https://en.m.wiktionary.org/wiki/%s", word);
    webkit_web_view_load_uri (WEBKIT_WEB_VIEW (window->webview), uri);

    g_free (uri);
}

static GtkWidget *
create_widget (gpointer item,
               gpointer user_data)
{
    GlobWord *word = GLOB_WORD (item);

    GtkWidget *row = gtk_list_box_row_new ();

    const gchar *text = glob_word_get_word (word);
    GtkWidget *label = gtk_label_new (text);

    gtk_container_add (GTK_CONTAINER (row), label);
    gtk_widget_show_all (row);

    return row;
}

static void
populate_finish (GObject      *source_object,
                 GAsyncResult *res,
                 gpointer      user_data)
{
    GlobAppWindow *win = GLOB_APP_WINDOW (user_data);
    GListModel *list;

    list = glob_search_get_matches_finish (GLOB_SEARCH (source_object), res, NULL);
    gtk_list_box_bind_model(GTK_LIST_BOX (win->listbox),
                            list, create_widget, win, NULL);
}

static void
entry_search_changed (GtkSearchEntry *entry,
                      GlobAppWindow  *win)
{
    if (gtk_entry_get_text_length (GTK_ENTRY (entry)) > 0)
    {
        const gchar *pattern;

        pattern = gtk_entry_get_text (GTK_ENTRY (entry));
        glob_search_get_matches_async (win->search, pattern, NULL, populate_finish, win);
    }
}

static void
glob_app_window_class_init (GlobAppWindowClass *klass)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 "/uk/me/bcowan/glob/glob.ui");

    gtk_widget_class_bind_template_child (widget_class, GlobAppWindow, header);
    gtk_widget_class_bind_template_child (widget_class, GlobAppWindow, webview);
    gtk_widget_class_bind_template_child (widget_class, GlobAppWindow, listbox);

    gtk_widget_class_bind_template_callback (widget_class, entry_search_changed);
    gtk_widget_class_bind_template_callback (widget_class, row_activated_cb);
}

static void
glob_app_window_init (GlobAppWindow *win)
{
    gtk_widget_init_template (GTK_WIDGET (win));
    gtk_window_maximize (GTK_WINDOW (win));

    win->search = glob_search_dup ();
    g_signal_connect (win->search, "finished",
                      G_CALLBACK (update_header), win);

    webkit_web_view_load_uri (WEBKIT_WEB_VIEW (win->webview),
                              "https://en.m.wiktionary.org/wiki/");
}

/**
 * glob_app_window_new:
 * @app: the application object
 *
 * Gets an application window with all its widgets setup for use.
 *
 * Returns: (transfer full) an application window
 */
GtkWidget *
glob_app_window_new (GlobApplication *app)
{
    return g_object_new (GLOB_TYPE_APP_WINDOW, "application", app, NULL);
}
