#include "glob-word.h"

enum
{
    PROP_0,
    PROP_WORD,
    PROP_LAST
};

static GParamSpec *pspecs[PROP_LAST];

struct _GlobWord
{
    GObject parent_instance;

    gchar *word;
};

G_DEFINE_TYPE (GlobWord, glob_word, G_TYPE_OBJECT)

static void
glob_word_get_property (GObject    *object,
                        guint       property_id,
                        GValue     *value,
                        GParamSpec *pspec)
{
    GlobWord *self = GLOB_WORD (object);

    switch (property_id)
    {
        case PROP_WORD:
            g_value_set_string (value, glob_word_get_word (self));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
glob_word_set_property (GObject      *object,
                        guint         property_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
    GlobWord *self = GLOB_WORD (object);

    switch (property_id)
    {
        case PROP_WORD:
            self->word = g_value_dup_string (value);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
glob_word_finalize (GObject *object)
{
    GlobWord *self = GLOB_WORD (object);

    g_free (self->word);

    G_OBJECT_CLASS (glob_word_parent_class)->finalize (object);
}

static void
glob_word_class_init (GlobWordClass *klass)
{
    GObjectClass *obj_class = G_OBJECT_CLASS (klass);

    obj_class->get_property = glob_word_get_property;
    obj_class->set_property = glob_word_set_property;
    obj_class->finalize = glob_word_finalize;

    pspecs[PROP_WORD] = g_param_spec_string ("word", "Word", "Dictionary word",
                                             "",
                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_properties (obj_class, PROP_LAST, pspecs);
}

static void
glob_word_init (GlobWord *self)
{

}

GlobWord *
glob_word_new (const gchar *word)
{
    return g_object_new (GLOB_TYPE_WORD,
                         "word", word,
                         NULL);
}

const gchar *
glob_word_get_word (GlobWord *self)
{
    g_return_val_if_fail (GLOB_IS_WORD (self), NULL);

    return self->word;
}
